#define IS_NUMERIC_KEY(key) ((key >= 0x30) && (key <= 0x39))
#define IS_ALPHA_KEY(key) ((key >= 0x41) && (key <= 0x5A))
#define IS_NUMPAD_KEY(key) ((key >= 0x60) && (key <= 0x6F))
#define IS_OTHER_KEY(key) ((key == VK_SPACE) || (key == VK_BACK) || (key == VK_DELETE))

#define KEY_CHECK(key) (IS_NUMERIC_KEY(key) || IS_ALPHA_KEY(key) || IS_NUMPAD_KEY(key) || IS_OTHER_KEY(key))

#ifdef __BORLANDC__
	#define _stricmp stricmp
#endif

#if DEBUG
	#pragma comment (lib, "../../blackbox/debug/blackbox.lib")
#else
	#pragma comment (lib, "../../blackbox/release/blackbox.lib")
	#ifndef NO_AGRESSION
		#include <AggressiveOptimize.h>
	#endif
#endif

#include "..\..\blackbox\BBApi.h"
//#include "C:\Documents and Settings\Tresni\My Documents\BB4Win\xoblite.h"
#include "Broambox.h"

extern "C" {
	DLL_EXPORT int beginPluginEx(HINSTANCE, HWND);
	DLL_EXPORT int beginPlugin(HINSTANCE);
	DLL_EXPORT void endPlugin(HINSTANCE);
	DLL_EXPORT LPCSTR pluginInfo(int);
};

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK EditProc(HWND,UINT,WPARAM,LPARAM);

Broambox *pBBox;

int beginPluginEx(HINSTANCE hInstance, HWND hWnd)
{
	pBBox = new Broambox(hInstance, hWnd);
	return pBBox->CreateBroambox();
}

int beginPlugin(HINSTANCE hInstance)
{
	return beginPluginEx(hInstance, NULL);
}

void endPlugin(HINSTANCE hInstance)
{
	pBBox->DestroyBroambox();
	delete(pBBox);
}

LPCSTR pluginInfo(int field)
{
	switch(field)
	{
		case PLUGIN_NAME: return pBBox->GetAppName();
		case PLUGIN_AUTHOR: return pBBox->GetAppAuthor();
		case PLUGIN_EMAIL: return pBBox->GetAppEmail();
		case PLUGIN_LINK: return pBBox->GetAppLink();
		case PLUGIN_RELEASE: return pBBox->GetAppRelease();
		case PLUGIN_VERSION: return pBBox->GetAppVersion();
		default:
		{
			char *temp = "";
			sprintf(temp, "%s %s (%s)", pBBox->GetAppName(), pBBox->GetAppVersion(), pBBox->GetAppRelease());
			return (LPCSTR)temp;
		}
		break;
	}
}
/*
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_NCHITTEST:
			return HTCAPTION;

		case WM_NCLBUTTONDOWN:
		{
			return pBBox->LButtonDown(hWnd, uMsg, wParam, lParam);
		}
		break;

		case WM_WINDOWPOSCHANGED:
		{
			if (!pBBox->InSlit())
				pBBox->MoveBroambox((WINDOWPOS*)lParam);
		}
		break;

		case WM_CREATE:
		{
			pBBox->CreateEditWindow(hWnd);
		}
		break;
		
		case WM_SETFOCUS:
		{
			pBBox->FocusEditWindow();
			return 0;
		}
		break;
		
		case WM_SIZE:
		{
			pBBox->SizeEditWindow();
			return 0;
		}
		break;

		case WM_PAINT:
		{
			pBBox->PaintBroambox();
			return 0;
		}
		break;

		case WM_CLOSE: { return 0; } break; // no ALT+F4

		case WM_CTLCOLOREDIT:
		{
			//gr
			SetTextColor((HDC)wParam, pBBox->editTextColor);
			SetBkMode((HDC)wParam, TRANSPARENT);
			return (LRESULT)GetStockObject(NULL_BRUSH);
		}
		break;

		case BB_BROADCAST:
		{
			char temp[MAX_LINE_LENGTH];
			strcpy(temp, (LPCSTR)lParam);

			if (!_stricmp(temp, "@BBShowPlugins"))
			{
				pBBox->ShowWindow();
			}
			else if(!_stricmp(temp, "@BBHidePlugins"))
			{
				pBBox->HideWindow();
			}
		}
		break;

		case BB_RECONFIGURE:
		{
			pBBox->ReadSettings();
			pBBox->SizeBroambox();
			pBBox->ReplaceFont();
		}
		break;
	}
	
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}
*/
LRESULT CALLBACK EditProc(HWND hText,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch(msg)
	{
		case WM_SETFOCUS:
		{
			SendMessage(hText,EM_SETSEL,0,-1);
			pBBox->ToggleSlitAutoHide(true);
		}
		break;

		case WM_KILLFOCUS:
		{
//			pBBox->ResetHistoryIndex();
			pBBox->ToggleSlitAutoHide(false);
		}
		break;

		case WM_NCHITTEST:
		{
			if((GetAsyncKeyState(VK_CONTROL) & 0x8000))
				return HTCAPTION;
		}
		break;

		case WM_NCRBUTTONUP:
			return pBBox->RButtonDown(hText, msg, wParam, lParam);
		
		case WM_KEYDOWN:
		{
			if(wParam==VK_DOWN)
			{
				pBBox->HistoryPrev();
				return 0;
			}
			if(wParam==VK_UP)
			{
				pBBox->HistoryNext();
				return 0;
			}
			if(KEY_CHECK(wParam))
				InvalidateRect(hText, NULL, true);
		}
		break;

		case WM_CHAR:
		{
			if(wParam==VK_RETURN)
			{
				pBBox->Execute();
				pBBox->ToggleSlitAutoHide(false);
				return 0;
			}
		}
		break;

		case WM_CLOSE: { return 0; } break; // no ALT+F4

		//gr
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			
			HDC hdc = BeginPaint(hText, &ps);
			RECT r;
			GetClientRect(hText, &r);
			
			HDC buf = CreateCompatibleDC(NULL);
			HGDIOBJ oldbuf = SelectObject(buf, CreateCompatibleBitmap(hdc, r.right, r.bottom));
			
			pBBox->PaintBroambox(buf, &r);
			
			CallWindowProc(pBBox->wpEditProc, hText, msg, (WPARAM)buf, lParam);
			
			BitBlt(hdc, 0,0, r.right, r.bottom, buf, 0, 0, SRCCOPY);
			
			DeleteObject(SelectObject(buf, oldbuf));
			DeleteDC(buf);
			
			EndPaint(hText, &ps);
			return 0;
		}
		break;

		case WM_CTLCOLOREDIT:
		{
			//gr
			SetTextColor((HDC)wParam, pBBox->textColor);
			SetBkMode((HDC)wParam, TRANSPARENT);
			return (LRESULT)GetStockObject(NULL_BRUSH);
		}
		break;

		case WM_ERASEBKGND:
			return TRUE;

		case BB_BROADCAST:
		{
			char temp[MAX_LINE_LENGTH];
			strcpy(temp, (LPCSTR)lParam);

			if (!_stricmp(temp, "@BBShowPlugins"))
			{
				pBBox->ShowWindow();
			}
			else if(!_stricmp(temp, "@BBHidePlugins"))
			{
				pBBox->HideWindow();
			}
			if(IsInString(temp, "@Broambox"))
			{
				char broam[MAX_LINE_LENGTH], command[MAX_LINE_LENGTH], arguments[MAX_LINE_LENGTH];
				char *tokens[2];
				tokens[0] = broam;
				tokens[1] = command;

				BBTokenize(temp, tokens, 2, arguments);
				
				if(_stricmp(command, "Toggle"))
				{
					if(_stricmp(arguments, "Slit"))
						pBBox->ToggleInSlit();
				}
			}
		}
		return 0;

		case BB_RECONFIGURE:
		{
			pBBox->ReadSettings();
			pBBox->SizeBroambox();
			pBBox->ReplaceFont();
		}
		return 0;
	}
	return CallWindowProc(pBBox->wpEditProc,hText,msg,wParam,lParam);
}